package com.common.im.ui.contact;


import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;


import com.common.im.adapter.ContactListAdapter;
import com.common.im.bean.ContactListBean;
import com.common.im.contract.ContactListFragmentContract;
import com.common.im.presenter.ContactListPresenter;
import com.common.im_ui.R;
import com.common.im_ui.databinding.FragmentContactListLayoutBinding;
import com.cooleshow.base.ui.fragment.BaseMVPFragment;
import com.cooleshow.base.utils.SizeUtils;
import com.cooleshow.base.widgets.EmptyViewLayout;
import com.mcxtzhang.indexlib.suspension.SuspensionDecoration;

import java.text.CollationKey;
import java.text.Collator;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import io.rong.imkit.RongIM;
import io.rong.imlib.model.Conversation;

/**
 * 联系人列表
 */
public class ContactPersonListFragment extends BaseMVPFragment<FragmentContactListLayoutBinding, ContactListPresenter> implements ContactListFragmentContract.ContactListFragmentView {

    private SuspensionDecoration mDecoration;
    private List<ContactListBean> mDatas = new ArrayList<>();
    private ContactListAdapter mListAdapter;
    private String searchCondition;
    private EmptyViewLayout mEmptyViewLayout;

    @Override
    protected FragmentContactListLayoutBinding getLayoutView() {
        return FragmentContactListLayoutBinding.inflate(getLayoutInflater());
    }

    @Override
    protected ContactListPresenter createPresenter() {
        return new ContactListPresenter();
    }

    @Override
    protected void initView(View rootView) {
        mListAdapter = new ContactListAdapter(R.layout.item_contact_list_layout);
        LinearLayoutManager layoutManager = new LinearLayoutManager(getContext());
        mViewBinding.recyclerView.setLayoutManager(layoutManager);
        mDecoration = new SuspensionDecoration(getContext(), mDatas);
        mDecoration.setColorTitleBg(Color.TRANSPARENT);
        mDecoration.setmTitleHeight(SizeUtils.dp2px(30));
        mViewBinding.recyclerView.addItemDecoration(mDecoration);
        mViewBinding.livLetters.setmPressedShowTextView(mViewBinding.tvHint)//设置HintTextView
                .setNeedRealIndex(false)//设置需要真实的索引
                .setmLayoutManager(layoutManager);//设置RecyclerView的LayoutManager
        mViewBinding.recyclerView.setAdapter(mListAdapter);
    }

    @Override
    protected void initData() {
        presenter.queryFriendList(searchCondition);
    }

    @Override
    public void getFriendListSuccess(List<ContactListBean> contactListBeans) {
        if (isDetached()) {
            return;
        }
        if (mListAdapter != null) {
            if (contactListBeans != null && contactListBeans.size() > 0) {
                mDatas = contactListBeans;
                mDecoration.setmDatas(mDatas);
                mViewBinding.livLetters.setmSourceDatas(mDatas)//设置数据
                        .invalidate();
                mListAdapter.setNewInstance(contactListBeans);
            } else {
                showEmptyView();
            }
        }
    }

    private void showEmptyView() {
        if (mEmptyViewLayout == null) {
            mEmptyViewLayout = new EmptyViewLayout(getContext());
        }
        mEmptyViewLayout.setContent(com.cooleshow.base.R.drawable.icon_empty_content, "暂无联系人");
        mListAdapter.setEmptyView(mEmptyViewLayout);
    }
}
