package com.rong.io.live.message;

import android.os.Parcel;
import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.UnsupportedEncodingException;

import io.rong.common.ParcelUtils;
import io.rong.imlib.MessageTag;
import io.rong.imlib.model.MessageContent;
import io.rong.imlib.model.UserInfo;

/**
 * Author by pq, Date on 2022/4/1.
 * 黑名单解除消息
 */
@MessageTag(value = "RC:UNBLOCK_BLACK_USER", flag = MessageTag.STATUS)
public class RCLiveBlackUserUnBlockMessage extends MessageContent {
    private String userId;


    public RCLiveBlackUserUnBlockMessage() {
    }

    public String getUserId() {
        return userId;
    }

    public void setUserId(String str) {
        this.userId = str;
    }

    public RCLiveBlackUserUnBlockMessage(byte[] data) {
        String jsonStr = null;
        try {
            jsonStr = new String(data, "UTF-8");
            Log.i("pq", "json:" + jsonStr);
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        try {
            JSONObject jsonObject = new JSONObject(jsonStr);

            if (jsonObject.has("user")) {
                setUserInfo(this.parseJsonToUserInfo(jsonObject.getJSONObject("user")));
            }

            if (jsonObject.has("content")) {
                JSONObject content = (JSONObject) jsonObject.opt("content");
                if (content != null) {
                    userId = content.optString("userId");
                }
            }
            Log.i("pq", "add black user userId:" + userId);
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    @Override
    public byte[] encode() {
        JSONObject jsonObj = new JSONObject();
        try {
            // 消息携带用户信息时, 自定义消息需添加下面代码
            JSONObject jsonUserInfo = this.getJSONUserInfo();
            if (jsonUserInfo != null) {
                jsonObj.putOpt("user", jsonUserInfo);
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }

        try {
            return jsonObj.toString().getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {
        dest.writeString(this.userId);
        ParcelUtils.writeToParcel(dest, getExtra());
        ParcelUtils.writeToParcel(dest, getUserInfo());
    }

    protected RCLiveBlackUserUnBlockMessage(Parcel in) {
        this.userId = in.readString();
        this.setExtra(ParcelUtils.readFromParcel(in));
        this.setUserInfo((UserInfo) ParcelUtils.readFromParcel(in, UserInfo.class));
    }

    public static final Creator<RCLiveBlackUserUnBlockMessage> CREATOR = new Creator<RCLiveBlackUserUnBlockMessage>() {
        @Override
        public RCLiveBlackUserUnBlockMessage createFromParcel(Parcel source) {
            return new RCLiveBlackUserUnBlockMessage(source);
        }

        @Override
        public RCLiveBlackUserUnBlockMessage[] newArray(int size) {
            return new RCLiveBlackUserUnBlockMessage[size];
        }
    };
}
