package com.rong.io.live.message;

import android.os.Parcel;
import android.text.TextUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.UnsupportedEncodingException;

import io.rong.common.ParcelUtils;
import io.rong.imlib.MessageTag;
import io.rong.imlib.model.MessageContent;
import io.rong.imlib.model.UserInfo;

/**
 * Author by pq, Date on 2022/4/1.
 * 学生端异常退出房间（如杀进程服务端发送的消息）
 */
@MessageTag(value = "RC:LookerLoginOut",flag = MessageTag.STATUS)
public class RCUserLogOutUnusualMessage extends MessageContent {
    private String userId  ;//退出用户ID
    private String userName   ;//退出用户名称

    public String getUserId() {
        return userId;
    }

    public void setUserId(String userId) {
        this.userId = userId;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public RCUserLogOutUnusualMessage() {
    }
    public RCUserLogOutUnusualMessage(byte[] data) {
        String jsonStr = null;
        try {
            jsonStr = new String(data, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        try {
            JSONObject jsonObject = new JSONObject(jsonStr);

            if (jsonObject.has("userId")) {
                userId = jsonObject.optString("userId");
            }
            if (jsonObject.has("userName")) {
                userName = jsonObject.optString("userName");
            }

            if (jsonObject.has("user")) {
                setUserInfo(this.parseJsonToUserInfo(jsonObject.getJSONObject("user")));
            }

        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    @Override
    public byte[] encode() {
        JSONObject jsonObj = new JSONObject();
        try {
            // 消息携带用户信息时, 自定义消息需添加下面代码
            JSONObject jsonUserInfo = this.getJSONUserInfo();
            if (!TextUtils.isEmpty(userId)) {
                jsonObj.putOpt("userId", userId);
            }

            if (!TextUtils.isEmpty(userName)) {
                jsonObj.putOpt("userName", userName);
            }

            if (jsonUserInfo != null) {
                jsonObj.putOpt("user", jsonUserInfo);
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }

        try {
            return jsonObj.toString().getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {
        dest.writeString(this.userId);
        dest.writeString(this.userName);
        ParcelUtils.writeToParcel(dest, getExtra());
        ParcelUtils.writeToParcel(dest, getUserInfo());
    }

    protected RCUserLogOutUnusualMessage(Parcel in) {
        this.userId = in.readString();
        this.userName = in.readString();
        this.setExtra(ParcelUtils.readFromParcel(in));
        this.setUserInfo((UserInfo)ParcelUtils.readFromParcel(in, UserInfo.class));
    }

    public static final Creator<RCUserLogOutUnusualMessage> CREATOR = new Creator<RCUserLogOutUnusualMessage>() {
        @Override
        public RCUserLogOutUnusualMessage createFromParcel(Parcel source) {
            return new RCUserLogOutUnusualMessage(source);
        }

        @Override
        public RCUserLogOutUnusualMessage[] newArray(int size) {
            return new RCUserLogOutUnusualMessage[size];
        }
    };
}
