package com.rong.io.live.message;

import android.os.Parcel;
import android.text.TextUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.UnsupportedEncodingException;

import io.rong.common.ParcelUtils;
import io.rong.imlib.MessageTag;
import io.rong.imlib.model.MessageContent;
import io.rong.imlib.model.UserInfo;

/**
 * Author by pq, Date on 2022/4/1.
 * 观众下麦发送消息--观众主动下麦或者被主播报下麦后学生端发送
 */
@MessageTag(value = "RC:Chatroom:downSeat",flag = MessageTag.STATUS)
public class RCUserMicLeaveMessage extends MessageContent {
    private String audienceId;
    private String audienceName;

    public String getAudienceId() {
        return audienceId;
    }

    public void setAudienceId(String audienceId) {
        this.audienceId = audienceId;
    }

    public String getAudienceName() {
        return audienceName;
    }

    public void setAudienceName(String audienceName) {
        this.audienceName = audienceName;
    }

    public RCUserMicLeaveMessage() {
    }
    public RCUserMicLeaveMessage(byte[] data) {
        String jsonStr = null;
        try {
            jsonStr = new String(data, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        try {
            JSONObject jsonObject = new JSONObject(jsonStr);

            if (jsonObject.has("user")) {
                setUserInfo(this.parseJsonToUserInfo(jsonObject.getJSONObject("user")));
            }

            if (jsonObject.has("audienceId")) {
                audienceId = jsonObject.optString("audienceId");
            }
            if (jsonObject.has("audienceName")) {
                audienceName = jsonObject.optString("audienceName");
            }

        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    @Override
    public byte[] encode() {
        JSONObject jsonObj = new JSONObject();
        try {
            // 消息携带用户信息时, 自定义消息需添加下面代码
            JSONObject jsonUserInfo = this.getJSONUserInfo();
            if (jsonUserInfo != null) {
                jsonObj.putOpt("user", jsonUserInfo);
            }

            if (!TextUtils.isEmpty(audienceId)) {
                jsonObj.putOpt("audienceId", audienceId);
            }
            if (!TextUtils.isEmpty(audienceName)) {
                jsonObj.putOpt("audienceName", audienceName);
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }

        try {
            return jsonObj.toString().getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {
        dest.writeString(this.audienceId);
        dest.writeString(this.audienceName);
        ParcelUtils.writeToParcel(dest, getExtra());
        ParcelUtils.writeToParcel(dest, getUserInfo());
    }

    protected RCUserMicLeaveMessage(Parcel in) {
        this.audienceId = in.readString();
        this.audienceName = in.readString();
        this.setExtra(ParcelUtils.readFromParcel(in));
        this.setUserInfo((UserInfo)ParcelUtils.readFromParcel(in, UserInfo.class));
    }

    public static final Creator<RCUserMicLeaveMessage> CREATOR = new Creator<RCUserMicLeaveMessage>() {
        @Override
        public RCUserMicLeaveMessage createFromParcel(Parcel source) {
            return new RCUserMicLeaveMessage(source);
        }

        @Override
        public RCUserMicLeaveMessage[] newArray(int size) {
            return new RCUserMicLeaveMessage[size];
        }
    };
}
