package com.rong.io.live.message;

import android.os.Parcel;
import android.text.TextUtils;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.UnsupportedEncodingException;

import io.rong.common.ParcelUtils;
import io.rong.imlib.MessageTag;
import io.rong.imlib.model.MessageContent;
import io.rong.imlib.model.UserInfo;

/**
 * Author by pq, Date on 2022/4/1.
 * 连麦控制
 */
@MessageTag(value = "RC:Chatroom:SeatsCtrl",flag = MessageTag.STATUS)
public class RCUserSeatsCtrlMessage extends MessageContent {
    private String userId;//操作者ID
    private String userName;//操作者名称
    private boolean seatBan ;//YES 禁止 NO 开启

    public String getUserId() {
        return userId;
    }

    public void setUserId(String userId) {
        this.userId = userId;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public boolean isSeatBan() {
        return seatBan;
    }

    public void setSeatBan(boolean seatBan) {
        this.seatBan = seatBan;
    }

    public RCUserSeatsCtrlMessage() {
    }
    public RCUserSeatsCtrlMessage(byte[] data) {
        String jsonStr = null;
        try {
            jsonStr = new String(data, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        try {
            JSONObject jsonObject = new JSONObject(jsonStr);

            if (jsonObject.has("userId")) {
                userId = jsonObject.optString("userId");
            }

            if (jsonObject.has("userName")) {
                userName = jsonObject.optString("userName");
            }

            if(jsonObject.has("seatBan")){
                seatBan = jsonObject.optBoolean("seatBan",false);
            }
            if (jsonObject.has("user")) {
                setUserInfo(this.parseJsonToUserInfo(jsonObject.getJSONObject("user")));
            }

        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    @Override
    public byte[] encode() {
        JSONObject jsonObj = new JSONObject();
        try {
            // 消息携带用户信息时, 自定义消息需添加下面代码
            JSONObject jsonUserInfo = this.getJSONUserInfo();

            if (!TextUtils.isEmpty(userId)) {
                jsonObj.putOpt("userId", userId);
            }

            if (!TextUtils.isEmpty(userName)) {
                jsonObj.putOpt("userName", userName);
            }

            jsonObj.putOpt("seatBan",seatBan);
            if (jsonUserInfo != null) {
                jsonObj.putOpt("user", jsonUserInfo);
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }

        try {
            return jsonObj.toString().getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {
        dest.writeString(this.userId);
        dest.writeString(this.userName);
        dest.writeByte(this.seatBan ? (byte) 1 : (byte) 0);
        ParcelUtils.writeToParcel(dest, getExtra());
        ParcelUtils.writeToParcel(dest, getUserInfo());
    }

    protected RCUserSeatsCtrlMessage(Parcel in) {
        this.userId = in.readString();
        this.userName = in.readString();
        this.seatBan = in.readByte() != 0;
        this.setExtra(ParcelUtils.readFromParcel(in));
        this.setUserInfo((UserInfo)ParcelUtils.readFromParcel(in, UserInfo.class));
    }

    public static final Creator<RCUserSeatsCtrlMessage> CREATOR = new Creator<RCUserSeatsCtrlMessage>() {
        @Override
        public RCUserSeatsCtrlMessage createFromParcel(Parcel source) {
            return new RCUserSeatsCtrlMessage(source);
        }

        @Override
        public RCUserSeatsCtrlMessage[] newArray(int size) {
            return new RCUserSeatsCtrlMessage[size];
        }
    };
}
