package com.rong.io.live.message;

import android.os.Parcel;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.UnsupportedEncodingException;

import io.rong.common.ParcelUtils;
import io.rong.imlib.MessageTag;
import io.rong.imlib.model.MessageContent;
import io.rong.imlib.model.UserInfo;

/**
 * Author by pq, Date on 2022/4/1.
 * 用户点赞消息
 */
@MessageTag(value = "RC:Chatroom:LikeCount", flag = MessageTag.STATUS)
public class RCUserSyncAddLikeCountMessage extends MessageContent {
    private int count;// 点赞数量


    public RCUserSyncAddLikeCountMessage() {
    }

    public int getCount() {
        return count;
    }

    public void setCount(int counts) {
        this.count = counts;
    }

    public RCUserSyncAddLikeCountMessage(byte[] data) {
        String jsonStr = null;
        try {
            jsonStr = new String(data, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        try {
            JSONObject jsonObject = new JSONObject(jsonStr);

            if (jsonObject.has("user")) {
                setUserInfo(this.parseJsonToUserInfo(jsonObject.getJSONObject("user")));
            }

            if (jsonObject.has("count")) {
                count = jsonObject.optInt("count");
            }


        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    @Override
    public byte[] encode() {
        JSONObject jsonObj = new JSONObject();
        try {
            // 消息携带用户信息时, 自定义消息需添加下面代码
            JSONObject jsonUserInfo = this.getJSONUserInfo();
            if (jsonUserInfo != null) {
                jsonObj.putOpt("user", jsonUserInfo);
            }

            jsonObj.putOpt("count", count);
        } catch (JSONException e) {
            e.printStackTrace();
        }

        try {
            return jsonObj.toString().getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {
        dest.writeInt(this.count);
        ParcelUtils.writeToParcel(dest, getExtra());
        ParcelUtils.writeToParcel(dest, getUserInfo());
    }

    protected RCUserSyncAddLikeCountMessage(Parcel in) {
        this.count = in.readInt();
        this.setExtra(ParcelUtils.readFromParcel(in));
        this.setUserInfo((UserInfo) ParcelUtils.readFromParcel(in, UserInfo.class));
    }

    public static final Creator<RCUserSyncAddLikeCountMessage> CREATOR = new Creator<RCUserSyncAddLikeCountMessage>() {
        @Override
        public RCUserSyncAddLikeCountMessage createFromParcel(Parcel source) {
            return new RCUserSyncAddLikeCountMessage(source);
        }

        @Override
        public RCUserSyncAddLikeCountMessage[] newArray(int size) {
            return new RCUserSyncAddLikeCountMessage[size];
        }
    };
}
