package com.cooleshow.student.ui.course;

import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.PopupWindow;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.alibaba.android.arouter.launcher.ARouter;
import com.bigkoo.pickerview.builder.OptionsPickerBuilder;
import com.bigkoo.pickerview.builder.TimePickerBuilder;
import com.bigkoo.pickerview.listener.CustomListener;
import com.bigkoo.pickerview.listener.OnOptionsSelectListener;
import com.bigkoo.pickerview.view.OptionsPickerView;
import com.bigkoo.pickerview.view.TimePickerView;
import com.chad.library.adapter.base.BaseQuickAdapter;
import com.chad.library.adapter.base.listener.OnItemClickListener;
import com.chad.library.adapter.base.listener.OnLoadMoreListener;
import com.cooleshow.base.constanst.Constants;
import com.cooleshow.base.router.RouterPath;
import com.cooleshow.base.ui.fragment.BaseMVPFragment;
import com.cooleshow.base.utils.LogUtils;
import com.cooleshow.base.utils.PopupUtil;
import com.cooleshow.base.utils.TimeUtils;
import com.cooleshow.base.widgets.EmptyViewLayout;
import com.cooleshow.student.R;
import com.cooleshow.student.adapter.CourseStatusFilterAdapter;
import com.cooleshow.student.adapter.PracticeCourseListAdapter;
import com.cooleshow.student.bean.CourseFilterStatusBean;
import com.cooleshow.student.bean.PracticeCourseListBean;
import com.cooleshow.student.bean.QuerySubjectBean;
import com.cooleshow.student.constants.CourseConstants;
import com.cooleshow.student.contract.MinePracticeCourseContract;
import com.cooleshow.student.databinding.FragmentPracticeCourseLayoutBinding;
import com.cooleshow.student.presenter.course.MinePracticeCoursePresenter;
import com.scwang.smart.refresh.layout.api.RefreshLayout;
import com.scwang.smart.refresh.layout.listener.OnRefreshListener;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * 创建日期：2022/5/30 10:29
 *
 * @author Ryan
 * 类说明：
 */
public class MinePracticeCourseFragment extends BaseMVPFragment<FragmentPracticeCourseLayoutBinding, MinePracticeCoursePresenter> implements MinePracticeCourseContract.MinePracticeCourseView, View.OnClickListener {
    private String currentCourseFilterStatus = CourseConstants.COURSE_FILTER_ALL;
    private String currentFilterDate;
    private Date currentSelectDate;
    private int currentSubjectId;
    private int currentPage;
    private boolean hasNext = true;
    private PracticeCourseListAdapter mAdapter;
    private EmptyViewLayout mEmptyView;
    private TimePickerView pvTime;
    private PopupWindow mPopupWindow;
    private ArrayList<CourseFilterStatusBean> mCourseFilterStatusBeans;
    private CourseStatusFilterAdapter mCourseStatusFilterAdapter;


    public static MinePracticeCourseFragment newInstance() {
        MinePracticeCourseFragment fragment = new MinePracticeCourseFragment();
        return fragment;
    }

    @Override
    public void onClick(View view) {
        switch (view.getId()) {
            case R.id.tv_time:
                //时间筛选
                showTimeSelectPicker();
                break;
            case R.id.tv_course_status:
                //课程状态筛选
                initPopu(mViewBinding.tvCourseStatus);
                break;
            case R.id.tv_agency:
                selectSubject();
                break;
        }
    }

    @Override
    protected FragmentPracticeCourseLayoutBinding getLayoutView() {
        return FragmentPracticeCourseLayoutBinding.inflate(getLayoutInflater());
    }

    @Override
    protected MinePracticeCoursePresenter createPresenter() {
        return new MinePracticeCoursePresenter();
    }

    @Override
    protected void initView(View rootView) {

        mViewBinding.tvTime.setOnClickListener(this);
        mViewBinding.tvCourseStatus.setOnClickListener(this);
        mViewBinding.tvAgency.setOnClickListener(this);
    }

    private void buildDefaultCourseStatusFilterList() {
        mCourseFilterStatusBeans = new ArrayList<>();
        mCourseFilterStatusBeans.add(new CourseFilterStatusBean(CourseConstants.COURSE_FILTER_ALL, "全部"));
        mCourseFilterStatusBeans.add(new CourseFilterStatusBean(CourseConstants.COURSE_FILTER_HAS_NOT_STARTED, "未开始"));
        mCourseFilterStatusBeans.add(new CourseFilterStatusBean(CourseConstants.COURSE_FILTER_IN_PROGRESS, "进行中"));
        mCourseFilterStatusBeans.add(new CourseFilterStatusBean(CourseConstants.COURSE_FILTER_COMPLETED, "已结束"));
    }

    private void reBuildFilter(Date date, String status) {
        currentSelectDate = date != null ? date : TimeUtils.getNowDate();
        String targetDateTimeStr = TimeUtils.date2String(currentSelectDate, TimeUtils.getSafeDateFormat("yyyy-MM"));
        if (TextUtils.equals(targetDateTimeStr, currentFilterDate) && TextUtils.equals(currentCourseFilterStatus, status)) {
            //防止重复条件触发
            return;
        }
        currentFilterDate = targetDateTimeStr;
        currentCourseFilterStatus = !TextUtils.isEmpty(status) ? status : CourseConstants.COURSE_FILTER_ALL;
        currentPage = 1;
        Log.e("askdjhkasdfs", "reBuildFilter: ");
        mViewBinding.tvTime.setText(currentFilterDate);
        queryCourse(true);
    }

    @Override
    protected void initData() {
        mViewBinding.refreshLayout.setOnRefreshListener(new OnRefreshListener() {
            @Override
            public void onRefresh(@NonNull RefreshLayout refreshLayout) {
                currentPage = 1;
                queryCourse(true);
            }
        });

        mAdapter = new PracticeCourseListAdapter(R.layout.item_sparring_course_list_layout);
        mAdapter.getLoadMoreModule().setOnLoadMoreListener(new OnLoadMoreListener() {
            @Override
            public void onLoadMore() {
                //上拉加载
                if (hasNext) {
                    currentPage++;
                    queryCourse(false);
                } else {
                    mAdapter.getLoadMoreModule().loadMoreEnd();
                }
            }
        });
        mAdapter.setOnItemClickListener(new OnItemClickListener() {
            @Override
            public void onItemClick(@NonNull BaseQuickAdapter<?, ?> adapter, @NonNull View view, int position) {
                if (position < mAdapter.getData().size()) {
                    PracticeCourseListBean.RowsBean rowsBean = mAdapter.getData().get(position);
                    if (rowsBean != null) {
                        ARouter.getInstance()
                                .build(RouterPath.CourseCenter.SPARRING_COURSE_DETAIL)
                                .withString(SparringCourseDetailActivity.COURSE_ID, rowsBean.courseId)
                                .withString(SparringCourseDetailActivity.COURSE_GROUP_ID, rowsBean.courseGoupId)
                                .withString(SparringCourseDetailActivity.STUDENT_ID, rowsBean.userId)
                                .navigation();
                    }
                }
            }
        });
        mViewBinding.recyclerView.setLayoutManager(new LinearLayoutManager(requireContext()));
        mViewBinding.recyclerView.setAdapter(mAdapter);
        buildDefaultCourseStatusFilterList();
        reBuildFilter(null, CourseConstants.COURSE_FILTER_ALL);
        presenter.querySubjectItem();
    }

    private void queryCourse(boolean isShowLoading) {
        //根据默认筛选条件查询
        LogUtils.i("pq", "currentFilterDate:" + currentFilterDate);
        LogUtils.i("pq", "currentCourseFilterStatus:" + currentCourseFilterStatus);
        LogUtils.i("pq", "currentSubjectId:" + currentSubjectId);
        LogUtils.i("pq", "currentPage:" + currentPage);
        presenter.queryPracticeCourse(isShowLoading, currentFilterDate, currentCourseFilterStatus, currentSubjectId, currentPage);
    }

    @Override
    public void onGetPracticeCourseSuccess(int page, PracticeCourseListBean sparringCourseBean) {
        if (isDetached()) {
            return;
        }
        if (sparringCourseBean != null) {
            if (page == 1) {
                //第一页
                mViewBinding.refreshLayout.finishRefresh();
                if (mAdapter != null) {
                    mAdapter.getData().clear();
                    mAdapter.notifyDataSetChanged();
                    if (sparringCourseBean.rows != null && sparringCourseBean.rows.size() > 0) {
                        checkHasNext(sparringCourseBean.rows.size());
                        mAdapter.setNewInstance(sparringCourseBean.rows);
                    } else {
                        showEmptyView();
                    }
                }
            } else {
                //加载更多
                if (mAdapter != null) {
                    if (sparringCourseBean.rows != null && sparringCourseBean.rows.size() > 0) {
                        mAdapter.getLoadMoreModule().loadMoreComplete();
                        checkHasNext(sparringCourseBean.rows.size());
                        mAdapter.addData(sparringCourseBean.rows);
                    } else {
                        mAdapter.getLoadMoreModule().loadMoreEnd();
                    }
                }
            }
        }
    }

    private void checkHasNext(int dataSize) {
        hasNext = dataSize >= Constants.DEFAULT_DATA_SIZE;
    }

    @Override
    public void onGetCourseError(int page) {
        if (isDetached()) {
            return;
        }
        if (page == 1) {
            mViewBinding.refreshLayout.finishRefresh();
        } else {
            if (mAdapter != null) {
                currentPage--;
                mAdapter.getLoadMoreModule().loadMoreFail();
            }
        }
    }

    private List<QuerySubjectBean> subjectBeanList = new ArrayList<>();

    @Override
    public void querySubjectItemSuccess(List<QuerySubjectBean> data) {
        subjectBeanList.clear();
        subjectBeanList.add(new QuerySubjectBean("全部声部",0));
        subjectBeanList.addAll(data);
    }

    private void showEmptyView() {
        if (mEmptyView == null) {
            mEmptyView = new EmptyViewLayout(getContext());
        }
        mEmptyView.setContent(com.cooleshow.base.R.drawable.icon_empty_course, "暂无课程~");
        mAdapter.setEmptyView(mEmptyView);
    }

    private void showTimeSelectPicker() {
        if (pvTime == null) {
            pvTime = new TimePickerBuilder(requireContext(), (date, v) -> {//选中事件回调
                reBuildFilter(date, currentCourseFilterStatus);
            }).setLayoutRes(com.cooleshow.base.R.layout.pickerview_default_layout, new CustomListener() {
                        @Override
                        public void customLayout(View v) {
                            //自定义布局中的控件初始化及事件处理
                            final TextView tvSubmit = (TextView) v.findViewById(com.cooleshow.base.R.id.tv_finish);
                            TextView ivCancel = (TextView) v.findViewById(com.cooleshow.base.R.id.tv_cancel);
                            tvSubmit.setOnClickListener(new View.OnClickListener() {
                                @Override
                                public void onClick(View v) {
                                    pvTime.returnData();
                                    pvTime.dismiss();
                                }
                            });
                            ivCancel.setOnClickListener(new View.OnClickListener() {
                                @Override
                                public void onClick(View v) {
                                    pvTime.dismiss();
                                }
                            });

                        }
                    })
                    .setLineSpacingMultiplier(2.5f)
                    .setType(new boolean[]{true, true, false, false, false, false})// 默认全部显示
                    .setTextColorCenter(getResources().getColor(com.cooleshow.base.R.color.color_1a1a1a))//设置选中项的颜色
                    .isDialog(false)//是否显示为对话框样式
                    .setLabel("年", "月", "", "", "", "")
                    .build();
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(currentSelectDate);
        pvTime.setDate(calendar);
        if (!pvTime.isShowing()) {
            pvTime.show();
        }
    }

    private void initPopu(View targetView) {
        if (mPopupWindow == null) {
            View popupContentView = LayoutInflater.from(requireContext()).inflate(com.cooleshow.base.R.layout.list_popu_layout, null);
            RecyclerView recyclerView = popupContentView.findViewById(R.id.recyclerView);
            View bg = popupContentView.findViewById(com.cooleshow.base.R.id.view);
            bg.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (mPopupWindow != null) {
                        mPopupWindow.dismiss();
                    }
                }
            });
            mCourseStatusFilterAdapter = new CourseStatusFilterAdapter(com.cooleshow.base.R.layout.notice_popu_list_item);
            mCourseStatusFilterAdapter.setOnItemClickListener(new OnItemClickListener() {
                @Override
                public void onItemClick(@NonNull BaseQuickAdapter<?, ?> adapter, @NonNull View view, int position) {
                    Object object = mCourseStatusFilterAdapter.getData().get(position);
                    if (object instanceof CourseFilterStatusBean) {
                        if (mCourseStatusFilterAdapter != null) {
                            mCourseStatusFilterAdapter.setSelect(position);
                        }
                        CourseFilterStatusBean filterStatusBean = (CourseFilterStatusBean) object;
                        mViewBinding.tvCourseStatus.setText(filterStatusBean.showText);
                        reBuildFilter(currentSelectDate, filterStatusBean.value);
                    }
                    mPopupWindow.dismiss();
                }
            });
            recyclerView.setLayoutManager(new LinearLayoutManager(requireContext()));
            recyclerView.setAdapter(mCourseStatusFilterAdapter);
            mCourseStatusFilterAdapter.setNewInstance(mCourseFilterStatusBeans);
            mPopupWindow = PopupUtil.createNoBackPopupWindow(popupContentView, requireContext(), ViewGroup.LayoutParams.MATCH_PARENT,
                    ViewGroup.LayoutParams.WRAP_CONTENT, true);
        }
        mPopupWindow.showAsDropDown(targetView);
    }
    private OptionsPickerView pvOptions;
    private void selectSubject() {
        pvOptions = new OptionsPickerBuilder(getContext(), new OnOptionsSelectListener() {
            @Override
            public void onOptionsSelect(int options1, int options2, int options3, View v) {
                currentSubjectId=subjectBeanList.get(options1).id;
                mViewBinding.tvAgency.setText(subjectBeanList.get(options1).name);
                queryCourse(true);

            }
        })
                .setLayoutRes(R.layout.pickerview_address_layout, new CustomListener() {
                    @Override
                    public void customLayout(View v) {
                        //自定义布局中的控件初始化及事件处理
                        final TextView tvSubmit = (TextView) v.findViewById(com.cooleshow.base.R.id.tv_finish);
                        TextView ivCancel = (TextView) v.findViewById(com.cooleshow.base.R.id.tv_cancel);
                        tvSubmit.setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                pvOptions.returnData();
                                pvOptions.dismiss();
                            }
                        });
                        ivCancel.setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View v) {
                                pvOptions.dismiss();
                            }
                        });

                    }
                })
                .isDialog(false)
                .build();
        pvOptions.setPicker(subjectBeanList);
        pvOptions.show();

    }
}
