package com.cooleshow.teacher.api;

import com.cooleshow.base.data.net.BaseResponse;
import com.cooleshow.teacher.bean.AccountDetailBean;
import com.cooleshow.teacher.bean.AccountTotalBean;
import com.cooleshow.teacher.bean.AddressBean;
import com.cooleshow.teacher.bean.CountOfUnreadBean;
import com.cooleshow.teacher.bean.CourseCommentListBean;
import com.cooleshow.teacher.bean.CourseTableDataBean;
import com.cooleshow.teacher.bean.HelpCenterContentBean;
import com.cooleshow.teacher.bean.HomeCountBean;
import com.cooleshow.teacher.bean.HomePageSheetMusicListBean;
import com.cooleshow.teacher.bean.HomeworkListBean;
import com.cooleshow.teacher.bean.LiveCourseListBean;
import com.cooleshow.teacher.bean.MineLiveCourseListBean;
import com.cooleshow.teacher.bean.MineVideoCourseListBean;
import com.cooleshow.teacher.bean.MusicSheetListBean;
import com.cooleshow.teacher.bean.MusicSheetShareBean;
import com.cooleshow.teacher.bean.PayTestBean;
import com.cooleshow.teacher.bean.ReceivedCommentListBean;
import com.cooleshow.teacher.bean.ResponseBankCardBean;
import com.cooleshow.teacher.bean.ResponseBindCardBean;
import com.cooleshow.teacher.bean.ResponseUserBankCardBean;
import com.cooleshow.teacher.bean.SparringCourseCommentBean;
import com.cooleshow.teacher.bean.SparringCourseHomeworkBean;
import com.cooleshow.teacher.bean.SparringCourseListBean;
import com.cooleshow.teacher.bean.SystemMessageBean;
import com.cooleshow.teacher.bean.TeachableInstrumentBean;
import com.cooleshow.teacher.bean.TeacherSelfStyleInfoBean;
import com.cooleshow.teacher.bean.TeacherUserInfo;
import com.cooleshow.teacher.bean.UserAccountBean;
import com.cooleshow.teacher.bean.VideoCourseListBean;
import com.cooleshow.teacher.bean.WithdrawalInfoBean;
import com.cooleshow.teacher.bean.WithdrawalRecordBean;
import com.cooleshow.usercenter.bean.SetDetailBean;
import com.cooleshow.usercenter.bean.UserInfo;

import java.util.List;
import java.util.Objects;

import io.reactivex.rxjava3.core.Observable;
import okhttp3.RequestBody;
import retrofit2.http.Body;
import retrofit2.http.GET;
import retrofit2.http.POST;
import retrofit2.http.Path;
import retrofit2.http.Query;

import static com.cooleshow.base.common.BaseConstant.AUTH_GROUP;
import static com.cooleshow.base.common.BaseConstant.CMS_SERVER;
import static com.cooleshow.base.common.BaseConstant.MALL_ADMIN_SERVER;
import static com.cooleshow.base.common.BaseConstant.MALL_PORTAL_SERVER;
import static com.cooleshow.base.common.BaseConstant.STUDENT_GROUP;
import static com.cooleshow.base.common.BaseConstant.TEACHER_GROUP;

/**
 * Author by pq, Date on 2022/4/21.
 */
public interface APIService {
    /**
     * 根据月份获取老师该月有课的日期
     *
     * @param
     * @return
     */
    @POST(TEACHER_GROUP + "courseSchedule/queryCourseSchedule")
    Observable<BaseResponse<List<String>>> getCourseScheduleDateByMonth(@Body RequestBody body);

    /**
     * 根据日期获取老师当日排课
     *
     * @param
     * @return
     */
    @POST(TEACHER_GROUP + "courseSchedule/queryCourseUser")
    Observable<BaseResponse<CourseTableDataBean>> getCourseSchedulesWithDate(@Body RequestBody body);

    /**
     * 查询陪练课
     *
     * @param
     * @return
     */
    @POST(TEACHER_GROUP + "courseSchedule/queryTeacherPracticeCourse")
    Observable<BaseResponse<SparringCourseListBean>> getSparringCourses(@Body RequestBody body);


    /**
     * 查询个人信息
     *
     * @param
     * @return
     */
    @GET(TEACHER_GROUP + "homework/detail/{courseId}")
    Observable<BaseResponse<SparringCourseHomeworkBean>> getSparringCourseHomework(@Path("courseId") String courseId);


    /**
     * 查询课程评价信息
     *
     * @param
     * @return
     */
    @POST(TEACHER_GROUP + "courseReplied/selectReplied")
    Observable<BaseResponse<SparringCourseCommentBean>> getSparringCourseComment(@Body RequestBody body);

    /**
     * 提交陪练课老师评价
     *
     * @param
     * @return
     */
    @POST(TEACHER_GROUP + "courseReplied/replied")
    Observable<BaseResponse<Object>> submitSparringCourseComment(@Body RequestBody body);

    /**
     * 提交陪练课老师布置作业
     *
     * @param
     * @return
     */
    @POST(TEACHER_GROUP + "homework/decorate")
    Observable<BaseResponse<Object>> submitSparringCourseHomework(@Body RequestBody body);

    /**
     * 提交陪练课老师作业点评
     *
     * @param
     * @return
     */
    @POST(TEACHER_GROUP + "homework/review")
    Observable<BaseResponse<Object>> submitSparringCourseHomeworkComment(@Body RequestBody body);

    /**
     * 查询直播课
     *
     * @param
     * @return
     */
    @POST(TEACHER_GROUP + "courseSchedule/queryTeacherLiveCourse")
    Observable<BaseResponse<LiveCourseListBean>> getLiveCourses(@Body RequestBody body);


    /**
     * 查询视频课
     *
     * @param
     * @return
     */
    @POST(TEACHER_GROUP + "videoLessonGroup/page")
    Observable<BaseResponse<VideoCourseListBean>> getVideoCourses(@Body RequestBody body);

    /**
     * 查询个人信息
     *
     * @param
     * @return
     */
    @GET(AUTH_GROUP + "api/queryUserInfo")
    Observable<BaseResponse<UserInfo>> getUserInfo();

    /**
     * 查询老师个人信息
     *
     * @param
     * @return
     */
    @GET(TEACHER_GROUP + "teacher/queryUserInfo")
    Observable<BaseResponse<TeacherUserInfo>> getTeacherUserInfo();

    /**
     * 查询老师个人风采
     *
     * @param
     * @return
     */
    @GET(TEACHER_GROUP + "teacher/queryTeacherStyle")
    Observable<BaseResponse<TeacherSelfStyleInfoBean>> getTeacherSelfStyle();

    /**
     * 查询我收到的评价
     *
     * @param
     * @return
     */
    @POST(TEACHER_GROUP + "courseReplied/myReplied")
    Observable<BaseResponse<ReceivedCommentListBean>> getReceivedComment(@Body RequestBody body);


    /**
     * 查询课后评价
     *
     * @param
     * @return
     */
    @POST(TEACHER_GROUP + "courseSchedule/queryTeacherPracticeCourse")
    Observable<BaseResponse<CourseCommentListBean>> getCourseCommentList(@Body RequestBody body);

    /**
     * 查询课后评价
     *
     * @param
     * @return
     */
    @POST(TEACHER_GROUP + "music/sheet/list")
    Observable<BaseResponse<HomePageSheetMusicListBean>> getSheetMusicList(@Body RequestBody body);

    /**
     * 乐器列表查询
     *
     * @return
     */
    @GET(TEACHER_GROUP + "subject/subjectSelect")
    Observable<BaseResponse<List<TeachableInstrumentBean>>> subjectSelect();

    /**
     * 我的-个人风采保存
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "teacher/saveTeacherStyle")
    Observable<BaseResponse<TeacherSelfStyleInfoBean>> saveTeacherStyle(@Body RequestBody body);

    /**
     * 获取设置详情
     *
     * @return
     */
    @GET(AUTH_GROUP + "user/getSetDetail")
    Observable<BaseResponse<SetDetailBean>> getSetDetail();

    /**
     * 修改设置信息
     *
     * @param body
     * @return
     */
    @POST(AUTH_GROUP + "user/submitSetDetail")
    Observable<BaseResponse<SetDetailBean>> submitSetDetail(@Body RequestBody body);

    /**
     * 校验用户手机号和密码
     *
     * @param phone
     * @param password
     * @return
     */
    @GET(AUTH_GROUP + "user/checkUser")
    Observable<BaseResponse<Object>> checkUser(@Query("phone") String phone, @Query("password") String password);

    /**
     * 修改手机号
     *
     * @param authCode
     * @param newPhone
     * @param oldPhone
     * @return
     */
    @GET(AUTH_GROUP + "user/updatePhone")
    Observable<BaseResponse<Object>> updatePhone(@Query("authCode") String authCode, @Query("newPhone") String newPhone, @Query("oldPhone") String oldPhone);

    /**
     * 手机验证码修改密码
     *
     * @param body
     * @return
     */
    @POST(AUTH_GROUP + "user/updatePassword")
    Observable<BaseResponse<Object>> updatePassword(@Body RequestBody body);

    /**
     * 发送登录短信验证码
     *
     * @param body mobile 手机号
     *             type:类型（PASSWD:修改密码，LOGIN:登录或注册,BANK:绑定银行卡,PHONE:修改手机号）
     * @return
     */
    @POST(TEACHER_GROUP + "code/sendSmsCode")
    Observable<BaseResponse<Object>> sendSmsCode(@Body RequestBody body);

    /**
     * 校验短信验证码
     *
     * @param body mobile 手机号
     *             type:类型（PASSWD:修改密码，LOGIN:登录或注册,BANK:绑定银行卡,PHONE:修改手机号）
     * @return
     */
    @POST(TEACHER_GROUP + "code/verifySmsCode")
    Observable<BaseResponse<Object>> verifySmsCode(@Body RequestBody body);

    /**
     * 实名认证
     *
     * @param body
     * @return
     */
    @POST(AUTH_GROUP + "user/realNameAuth")
    Observable<BaseResponse<Object>> realNameAuth(@Body RequestBody body);

    /**
     * 获取所有收货地址
     *
     * @return
     */
    @GET(MALL_PORTAL_SERVER + "member/address/list")
    Observable<BaseResponse<List<AddressBean>>> companyAddressList();

    /**
     * 添加收货地址
     *
     * @param body
     * @return
     */
    @POST(MALL_PORTAL_SERVER + "member/address/add")
    Observable<BaseResponse<Object>> createNewAddress(@Body RequestBody body);

    /**
     * 修改地址
     *
     * @param id
     * @param body
     * @return
     */
    @POST(MALL_PORTAL_SERVER + "member/address/update/{id}")
    Observable<BaseResponse<Object>> updateAddress(@Path("id") int id, @Body RequestBody body);

    /**
     * 意见反馈
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "sysSuggestion/add")
    Observable<BaseResponse<Object>> addSuggestion(@Body RequestBody body);

    /**
     * 银行卡查询
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "userBankCard/page")
    Observable<BaseResponse<ResponseBankCardBean>> queryUserBankCard(@Body RequestBody body);

    /**
     * 用户绑卡
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "userBankCard/bankCard")
    Observable<BaseResponse<ResponseBindCardBean>> bindBankCard(@Body RequestBody body);

    /**
     * 用户解绑银行卡
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "userBankCard/unBind")
    Observable<BaseResponse<ResponseBindCardBean>> unbindBankCard(@Body RequestBody body);


    /**
     * 查询银行
     *
     * @return
     */
    @POST(TEACHER_GROUP + "userBankCard/bankList")
    Observable<BaseResponse<List<ResponseUserBankCardBean>>> userBankCard();

    /**
     * 分页查询直播课课程组列表
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "courseGroup/queryPageCourseGroup")
    Observable<BaseResponse<MineLiveCourseListBean>> queryPageCourseGroup(@Body RequestBody body);

    /**
     * 查询视频课
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "videoLesson/page")
    Observable<BaseResponse<MineVideoCourseListBean>> queryVideoCourseGroup(@Body RequestBody body);

    /**
     * 分页查询分页查询
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "music/sheet/list")
    Observable<BaseResponse<MusicSheetListBean>> queryMusicList(@Body RequestBody body);

    /**
     * 乐谱分享
     *
     * @return
     */
    @GET(TEACHER_GROUP + "music/sheet/share")
    Observable<BaseResponse<MusicSheetShareBean>> musicSheetShare();

    /**
     * 课后作业
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "homework/list")
    Observable<BaseResponse<HomeworkListBean>> queryHomeworkList(@Body RequestBody body);

    /**
     * 我的收入
     *
     * @return
     */
    @GET(TEACHER_GROUP + "userAccount/detail")
    Observable<BaseResponse<AccountDetailBean>> getUserAccountDetail();

    /**
     * 收入列表
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "userAccount/page")
    Observable<BaseResponse<UserAccountBean>> userAccountPage(@Body RequestBody body);

    /**
     * 查询提现页面信息
     *
     * @return
     */
    @POST(TEACHER_GROUP + "userWithdrawal/getWithdrawalInfo")
    Observable<BaseResponse<WithdrawalInfoBean>> getWithdrawalInfo();

    /**
     * 发起提现
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "userWithdrawal/withdrawal")
    Observable<BaseResponse<Object>> doWithdrawal(@Body RequestBody body);

    /**
     * 提现记录
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "userWithdrawal/withdrawalPage")
    Observable<BaseResponse<WithdrawalRecordBean>> getWithdrawalPage(@Body RequestBody body);

    /**
     * 获取所有消息列表
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "sysMessage/list")
    Observable<BaseResponse<SystemMessageBean>> sysMessageList(@Body RequestBody body);

    /**
     * 一键已读
     *
     * @return
     */
    @POST(TEACHER_GROUP + "sysMessage/batchSetRead")
    Observable<BaseResponse<Object>> batchSetRead();

    /**
     * 设置已读
     *
     * @return
     */
    @POST(TEACHER_GROUP + "sysMessage/setRead/{id}")
    Observable<BaseResponse<Object>> setCurrentRead(@Path("id") long id);

    /**
     * 查询用户未读消息条数
     *
     * @return
     */
    @GET(TEACHER_GROUP + "sysMessage/queryCountOfUnread")
    Observable<BaseResponse<List<CountOfUnreadBean>>> queryCountOfUnread();

    /**
     * 首页统计数据
     *
     * @return
     */
    @GET(TEACHER_GROUP + "home/count")
    Observable<BaseResponse<HomeCountBean>> getHomeCount();

    /**
     * 测试订单支付
     *
     * @return
     */
    @POST(STUDENT_GROUP + "userOrder/orderPay")
    Observable<BaseResponse<PayTestBean>> orderPay(@Body RequestBody body);

    /**
     * 首页统计数据
     *
     * @return
     */
    @POST(CMS_SERVER + "helpCenterContent/list")
    Observable<BaseResponse<HelpCenterContentBean>> helpCenterContentList(@Body RequestBody body);

    /**
     *
     * @param body
     * @return
     */
    @POST(TEACHER_GROUP + "userAccount/accountTotal")
    Observable<BaseResponse<AccountTotalBean>> accountTotal(@Body RequestBody body);
}
