package com.cooleshow.teacher.ui.homepage;

import android.graphics.Typeface;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.alibaba.android.arouter.facade.annotation.Route;
import com.common.im.adapter.MessagePagerAdapter;
import com.cooleshow.base.router.RouterPath;
import com.cooleshow.base.ui.activity.BaseMVPActivity;
import com.cooleshow.base.utils.GlideUtils;
import com.cooleshow.base.utils.LogUtils;
import com.cooleshow.base.utils.SizeUtils;
import com.cooleshow.base.utils.UiUtils;
import com.cooleshow.base.utils.Utils;
import com.cooleshow.teacher.R;
import com.cooleshow.teacher.adapter.HomePagerAdapter;
import com.cooleshow.teacher.bean.TeacherUserInfo;
import com.cooleshow.teacher.constants.TeacherInfoConstants;
import com.cooleshow.teacher.contract.HomePageContract;
import com.cooleshow.teacher.databinding.ActivityHomePageLayoutBinding;
import com.cooleshow.teacher.presenter.homePage.HomePagePresenter;
import com.google.android.material.tabs.TabLayout;
import com.google.android.material.tabs.TabLayoutMediator;

import org.w3c.dom.Text;

import java.util.ArrayList;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;

/**
 * Author by pq, Date on 2022/5/9.
 * 个人主页
 */
@Route(path = RouterPath.MineCenter.HOME_PAGE)
public class HomePageActivity extends BaseMVPActivity<ActivityHomePageLayoutBinding, HomePagePresenter> implements HomePageContract.HomePageView, View.OnClickListener {
    public static final String[] titles = new String[]{"个人风采", "陪练课", "直播课", "视频课", "乐谱"};
    private ArrayList<Fragment> fragments = new ArrayList<>();
    private SheetMusicPageFragment mMusicPageFragment;

    @Override
    protected void initView() {
        Utils.setHeadView(viewBinding.viewStatusBar, this, 0);
        viewBinding.ivBack.setOnClickListener(this);
    }

    @Override
    public void initData() {
        super.initData();
        TabLayoutMediator tabLayoutMediator = new TabLayoutMediator(viewBinding.tabLayout, viewBinding.viewPager, new TabLayoutMediator.TabConfigurationStrategy() {
            @Override
            public void onConfigureTab(@NonNull TabLayout.Tab tab, int position) {
                createTab(tab, titles[position]);
            }
        });
        viewBinding.tabLayout.addOnTabSelectedListener(new TabLayout.OnTabSelectedListener() {
            @Override
            public void onTabSelected(TabLayout.Tab tab) {
                if (tab != null && tab.getCustomView() != null) {
                    View customView = tab.getCustomView();
                    TextView tv_text = customView.findViewById(R.id.tv_text);
                    tv_text.setTextColor(getResources().getColor(com.cooleshow.base.R.color.color_333333));
                    tv_text.setTypeface(Typeface.defaultFromStyle(Typeface.BOLD));
                    tv_text.setTextSize(17);
                }
            }

            @Override
            public void onTabUnselected(TabLayout.Tab tab) {
                if (tab != null && tab.getCustomView() != null) {
                    View customView = tab.getCustomView();
                    TextView tv_text = customView.findViewById(R.id.tv_text);
                    tv_text.setTextColor(getResources().getColor(com.cooleshow.base.R.color.color_666666));
                    tv_text.setTypeface(Typeface.defaultFromStyle(Typeface.NORMAL));
                    tv_text.setTextSize(15);
                }
            }

            @Override
            public void onTabReselected(TabLayout.Tab tab) {
            }
        });
        MineStyleFragment mineStyleFragment = new MineStyleFragment();
        SparringCoursePageFragment sparringCoursePageFragment = new SparringCoursePageFragment();
        LiveCoursePageFragment liveCoursePageFragment = new LiveCoursePageFragment();
        VideoCoursePageFragment videoCoursePageFragment = new VideoCoursePageFragment();
        mMusicPageFragment = new SheetMusicPageFragment();
        fragments.add(mineStyleFragment);
        fragments.add(sparringCoursePageFragment);
        fragments.add(liveCoursePageFragment);
        fragments.add(videoCoursePageFragment);
        fragments.add(mMusicPageFragment);
        HomePagerAdapter homePagerAdapter = new HomePagerAdapter(this);
        homePagerAdapter.setData(fragments);
        viewBinding.viewPager.setOffscreenPageLimit(fragments.size());
        viewBinding.viewPager.setAdapter(homePagerAdapter);
        tabLayoutMediator.attach();
    }

    @Override
    protected ActivityHomePageLayoutBinding getLayoutView() {
        return ActivityHomePageLayoutBinding.inflate(getLayoutInflater());
    }

    @Override
    protected HomePagePresenter createPresenter() {
        return new HomePagePresenter();
    }

    @Override
    protected void onResume() {
        super.onResume();
        presenter.getTeacherUserInfo();
    }

    @Override
    public void onClick(View v) {
        int id = v.getId();
        if (id == R.id.iv_back) {
            finish();
            return;
        }
    }

    private TabLayout.Tab createTab(TabLayout.Tab tab, String text) {
        tab.setCustomView(R.layout.view_homepage_tab_layout);
        View customView = tab.getCustomView();
        if (customView != null) {
            TextView tv_text = customView.findViewById(R.id.tv_text);
            tv_text.setText(text);
        }
        return tab;
    }

    /**
     * 获取老师信息成功
     *
     * @param info
     */
    @Override
    public void getTeacherInfoSuccess(TeacherUserInfo info) {
        if (isFinishing() || isDestroyed()) {
            return;
        }
        if (mMusicPageFragment != null) {
            mMusicPageFragment.setMusicPersonCertStatus(info.musicianAuthStatus);
        }
        //星级
        viewBinding.ratingBar.setRating(info.starGrade);
        //粉丝数
        viewBinding.tvFansNum.setText(info.fansNum);
        //已上课时数
        viewBinding.tvCourseNumCompleted.setText(info.expTime);
        //头像
        GlideUtils.INSTANCE.loadImage(this, info.heardUrl, viewBinding.ivAvatar, R.drawable.icon_teacher_default_head);
        //昵称
        viewBinding.tvNickname.setText(info.username);
        if (TextUtils.equals(info.entryStatus, TeacherInfoConstants.ENTRY_STATUS_PASS)) {
            //入驻审核通过
            viewBinding.ivTeacherEntryTag.setVisibility(View.VISIBLE);
        } else {
            //未通过或者未申请
            viewBinding.ivTeacherEntryTag.setVisibility(View.GONE);
        }

        //音乐人认证状态
        if (TextUtils.equals(info.musicianAuthStatus, TeacherInfoConstants.MUSICIAN_STATUS_PASS)) {
            viewBinding.ivTeacherMusicPersonTag.setVisibility(View.VISIBLE);
        } else {
            viewBinding.ivTeacherMusicPersonTag.setVisibility(View.GONE);
        }
        //subjectName
        try {
            viewBinding.llSubjectContainer.removeAllViews();
            String[] subjectNames = UiUtils.splitSubjectName(info.subjectName);
            for (int i = 0; i < subjectNames.length; i++) {
                createSubjectView(subjectNames[i]);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void createSubjectView(String text) {
        TextView textView = (TextView) LayoutInflater.from(this).inflate(com.cooleshow.base.R.layout.view_subject_item_layout, viewBinding.llSubjectContainer, false);
        textView.setText(text);
        LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        if (viewBinding.llSubjectContainer.getChildCount() >= 1) {
            params.setMargins(SizeUtils.dp2px(4), 0, 0, 0);
        }
        viewBinding.llSubjectContainer.addView(textView, params);
    }
}
