package com.cooleshow.teacher.ui.score;

import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;

import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.viewpager.widget.ViewPager;

import com.alibaba.android.arouter.facade.annotation.Route;
import com.cooleshow.base.router.RouterPath;
import com.cooleshow.base.ui.activity.BaseMVPActivity;
import com.cooleshow.teacher.R;
import com.cooleshow.teacher.adapter.MineLiveCoursePagerAdapter;
import com.cooleshow.teacher.bean.MusicSheetShareBean;
import com.cooleshow.teacher.contract.MineScoreActContract;
import com.cooleshow.teacher.databinding.ActivityMineScoreBinding;
import com.cooleshow.teacher.presenter.score.MineScoreActPresenter;
import com.cooleshow.teacher.presenter.score.MineScorePresenter;
import com.cooleshow.teacher.widgets.DialogUtils;
import com.google.android.material.tabs.TabLayout;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import io.rong.imkit.utils.StatusBarUtil;

/**
 * 创建日期：2022/5/20 16:34
 *
 * @author Ryan
 * 类说明：
 */
@Route(path = RouterPath.ScoreCenter.TEACHER_MINE_SCORE)
public class MineScoreActivity extends BaseMVPActivity<ActivityMineScoreBinding, MineScoreActPresenter> implements MineScoreActContract.MineScoreActView, View.OnClickListener {
    @Override
    public void onClick(View view) {
        switch (view.getId()) {
            case R.id.tv_right:
                presenter.musicSheetShare();
                break;
        }

    }

    private TabLayout tabLayout;
    private ViewPager viewPager;

    @Override
    protected void initView() {
        initMidTitleToolBar(viewBinding.toolbarInclude.toolbar, "我的曲谱");
        viewBinding.toolbarInclude.tvRight.setBackgroundResource(R.drawable.icon_toolbar_share);
        viewBinding.toolbarInclude.tvRight.setVisibility(View.VISIBLE);
        viewBinding.toolbarInclude.tvRight.setOnClickListener(this);
        tabLayout = viewBinding.tablayout;
        viewPager = viewBinding.viewpager;
        viewPager.setCurrentItem(0);
    }

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        StatusBarUtil.setStatusBarDarkTheme(this, true);
    }

    @Override
    public void initData() {
        super.initData();
        initTabLayoutAndViewPager();
    }

    @Override
    protected ActivityMineScoreBinding getLayoutView() {
        return ActivityMineScoreBinding.inflate(getLayoutInflater());
    }

    @Override
    protected MineScoreActPresenter createPresenter() {
        return new MineScoreActPresenter();
    }

    private List<String> titles = new ArrayList<String>(Arrays.asList("已上架", "审核中", "审核失败"));
    private List<String> auditStatusList = new ArrayList<String>(Arrays.asList("DOING", "PASS", "UNPASS"));
    private List<Fragment> fragments = new ArrayList<>();

    private void initTabLayoutAndViewPager() {
        tabLayout.removeAllTabs();
        fragments.clear();
        for (int i = 0; i < titles.size(); i++) {
            tabLayout.addTab(tabLayout.newTab().setText(titles.get(i)));
            Fragment adFragment = MineScoreFragment.newInstance(auditStatusList.get(i));
            fragments.add(adFragment);
        }
        viewPager.setAdapter(new MineLiveCoursePagerAdapter(getSupportFragmentManager(), fragments, titles));
        tabLayout.setupWithViewPager(viewPager);
    }

    @Override
    public void musicSheetShareSuccess(MusicSheetShareBean data) {
        showScoreShareDialog(data);
    }

    private void showScoreShareDialog(MusicSheetShareBean data) {
        DialogUtils.showScoreShare(this,data, new DialogUtils.ActionDialogCallback() {
            @Override
            public void onConfirmClick() {

            }

            @Override
            public void onCancelClick() {

            }
        });
    }
}
