package com.cooleshow.teacher.ui.web;

import static com.cooleshow.base.common.WebConstants.WEB_URL;

import android.view.KeyEvent;
import android.view.View;
import android.webkit.WebView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.alibaba.android.arouter.facade.annotation.Route;
import com.cooleshow.base.router.RouterPath;
import com.cooleshow.base.ui.activity.BaseMVPActivity;
import com.cooleshow.base.widgets.AgentWebClient;
import com.cooleshow.teacher.databinding.ActivityAgentWebBinding;
import com.cooleshow.teacher.presenter.web.AgentWebPresenter;
import com.cooleshow.usercenter.helper.UserHelper;
import com.just.agentweb.AgentWeb;
import com.just.agentweb.WebChromeClient;

/**
 * 创建日期：2022/6/5 13:32
 *
 * @author Ryan
 * 类说明：
 */

public class AgentWebActivity extends BaseMVPActivity<ActivityAgentWebBinding, AgentWebPresenter> {
    private AgentWeb mAgentWeb;
    private LinearLayout container;
    private TextView tv_title;
    private AgentWebClient mWebClient;

    @Override
    protected void initView() {
        initMidTitleToolBar(viewBinding.toolbarInclude.toolbar, "酷乐秀");
        container = viewBinding.container;
        tv_title = viewBinding.toolbarInclude.title;
        viewBinding.toolbarInclude.ivBack.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                goBack();
            }
        });
        String linkurl = getIntent().getStringExtra(WEB_URL);
        mWebClient = new AgentWebClient();
        linkurl=loadUrl(linkurl);
        mAgentWeb = AgentWeb.with(this)
                .setAgentWebParent((LinearLayout) container, new LinearLayout.LayoutParams(-1, -1))
                .useDefaultIndicator()
                .setWebChromeClient(mWebChromeClient)
                .setWebViewClient(mWebClient)
                .createAgentWeb()
                .ready()
                .go(linkurl);

    }

    private String loadUrl(String url) {
        String userToken = UserHelper.getUserToken();
        mWebClient.setAuthorization(userToken);
        boolean status = url.contains("?");
        if (status) {
            url = (url + "&Authorization=" + userToken);
        } else {
            url = (url + "?Authorization=" + userToken);
        }
        return url;
    }


    private void goBack() {
        if (!mAgentWeb.back()) {
            finish();
        }
    }

    @Override
    protected void onPause() {
        mAgentWeb.getWebLifeCycle().onPause();
        super.onPause();

    }

    @Override
    protected void onResume() {
        mAgentWeb.getWebLifeCycle().onResume();
        super.onResume();
    }

    @Override
    public void onDestroy() {
        mAgentWeb.getWebLifeCycle().onDestroy();
        super.onDestroy();
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (mAgentWeb.handleKeyEvent(keyCode, event)) {
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    private WebChromeClient mWebChromeClient = new WebChromeClient() {
        @Override
        public void onReceivedTitle(WebView view, String title) {
            super.onReceivedTitle(view, title);
            tv_title.setText(title);
        }
    };

    @Override
    protected ActivityAgentWebBinding getLayoutView() {
        return ActivityAgentWebBinding.inflate(getLayoutInflater());
    }

    @Override
    protected AgentWebPresenter createPresenter() {
        return new AgentWebPresenter();
    }
}
